function VariableTimeSfun(block)
%VariableTimeSfun
%
% This is a level-2 m-code S-Function that performs a zero order
% hold sampling of an input signal.
% The sample time of the block is variable and is based on the value of a
% second input signal. (This value must be positive.)

% Author: Phil Goddard (phil@goddardconsulting.ca)
% Date: Q2, 2011

%%
%% The setup method is used to setup the basic attributes of the
%% S-function such as ports, parameters, etc. Do not add any other
%% calls to the main body of the function.
%%
setup(block);

%endfunction

%% Function: setup ===================================================
%% Abstract:
%%   Set up the S-function block's basic characteristics such as:
%%   - Input ports
%%   - Output ports
%%   - Dialog parameters
%%   - Options
%%
%%   Required         : Yes
%%   C-Mex counterpart: mdlInitializeSizes
%%
function setup(block)

% Register number of ports
block.NumInputPorts  = 2;
block.NumOutputPorts = 1;

% Setup port properties to be inherited or dynamic
%block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;

% Override input port properties
for idx = 1:block.NumInputPorts
    block.InputPort(idx).DatatypeID  = 0;  % double
    block.InputPort(idx).Complexity  = 'Real';
    block.InputPort(idx).SamplingMode  = 0; % no frames
    block.InputPort(idx).DirectFeedthrough = true;
end

% Override output port properties
block.OutputPort(1).DatatypeID  = 0; % double
block.OutputPort(1).Complexity  = 'Real';
block.OutputPort(1).SamplingMode  = 0; % no frames

% Register parameters
block.NumDialogPrms = 0;

% Register sample times
%  [0 offset]            : Continuous sample time
%  [positive_num offset] : Discrete sample time
%
%  [-1, 0]               : Inherited sample time
%  [-2, 0]               : Variable sample time
block.SampleTimes = [-2 0];

%% -----------------------------------------------------------------
%% Options
%% -----------------------------------------------------------------
% Specify if Accelerator should use TLC or call back into
% M-file
block.SetAccelRunOnTLC(false);

%% -----------------------------------------------------------------
%% The M-file S-function uses an internal registry for all
%% block methods. You should register all relevant methods
%% (optional and required) as illustrated below. You may choose
%% any suitable name for the methods and implement these methods
%% as local functions within the same file.
%% -----------------------------------------------------------------

block.RegBlockMethod('Outputs', @Outputs);

%% -------------------------------------------------------------------
%% The local functions
%% -------------------------------------------------------------------

function Outputs(block)

% Only change the output if this is a mjor time step
if block.IsMajorTimeStep
    % sample and output the input
    block.OutputPort(1).Data = block.InputPort(1).Data;
    
    % set the next sample time, which must have a positive step size
    if block.InputPort(2).Data < eps
        error('The requested step size has become negative.');
    else
        block.NextTimeHit = block.CurrentTime + block.InputPort(2).Data;
    end
end


